package com.example.PhidgetLCDExample;

import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;

import com.phidget22.*;

public class LCDExample extends Activity {

	LCD ch;
	EditText text1;
	EditText text2;
	SeekBar backlightBar;
	SeekBar contrastBar;

	Toast errToast;

	int minDataInterval;

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        //Hide device information and settings until one is attached
		LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);
		settingsAndData.setVisibility(LinearLayout.GONE);

		text1 = ((EditText)findViewById(R.id.text1));
		text1.setOnEditorActionListener(new textChangedListener());
		text1.setOnFocusChangeListener(new focusChangedListener());

		text2 = ((EditText)findViewById(R.id.text2));
		text2.setOnEditorActionListener(new textChangedListener());
		text2.setOnFocusChangeListener(new focusChangedListener());

		((EditText)findViewById(R.id.text1)).setText("", TextView.BufferType.EDITABLE);
		((EditText)findViewById(R.id.text2)).setText("", TextView.BufferType.EDITABLE);

		//set backlight bar functionality
		backlightBar = (SeekBar) findViewById(R.id.backlightBar);
		backlightBar.setOnSeekBarChangeListener(new backlightChangeListener());

		//set contrast bar functionality
		contrastBar = (SeekBar) findViewById(R.id.contrastBar);
		contrastBar.setOnSeekBarChangeListener(new contrastChangeListener());

        try
        {
        	ch = new LCD();

        	//Allow direct USB connection of Phidgets
			if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
                com.phidget22.usb.Manager.Initialize(this);

			//Enable server discovery to list remote Phidgets
			this.getSystemService(Context.NSD_SERVICE);
			Net.enableServerDiscovery(ServerType.DEVICE_REMOTE);

			ch.addAttachListener(new AttachListener() {
				public void onAttach(final AttachEvent attachEvent) {
				    AttachEventHandler handler = new AttachEventHandler(ch);
                    synchronized(handler)
					{
						runOnUiThread(handler);
						try {
							handler.wait();
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			});

			ch.addDetachListener(new DetachListener() {
				public void onDetach(final DetachEvent detachEvent) {
                    DetachEventHandler handler = new DetachEventHandler(ch);
                    synchronized(handler)
					{
						runOnUiThread(handler);
						try {
							handler.wait();
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			});

			ch.addErrorListener(new ErrorListener() {
				public void onError(final ErrorEvent errorEvent) {
					ErrorEventHandler handler = new ErrorEventHandler(ch, errorEvent);
					runOnUiThread(handler);
				}
			});

			ch.open();
        } catch (PhidgetException pe) {
	        pe.printStackTrace();
		}

    }

	@Override
	protected void onResume() {
		super.onResume();

		text1.setText("");
		text2.setText("");
	}

	public static String padRight(String s, int n) {
		return String.format("%1$-" + n + "s", s);
	}

	private void updateScreen() {
		try {
			ch.writeText(LCDFont.DIMENSIONS_5X8, 0, 0,
					padRight(text1.getText().toString(), ch.getWidth()));
			ch.writeText(LCDFont.DIMENSIONS_5X8, 0, 1,
					padRight(text2.getText().toString(), ch.getWidth()));
			ch.flush();
		} catch (PhidgetException e) {
			e.printStackTrace();
		}
	}

	private class textChangedListener implements EditText.OnEditorActionListener {
		@Override
		public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
			updateScreen();
			return true;
		}
	}

	private class focusChangedListener implements EditText.OnFocusChangeListener {
		@Override
		public void onFocusChange(View v, boolean hasFocus) {
			updateScreen();
		}
	}

	private class backlightChangeListener implements SeekBar.OnSeekBarChangeListener {
		public void onProgressChanged(SeekBar seekBar, int progress,
									  boolean fromUser) {
			try {
				TextView backlightTxt = (TextView) findViewById(R.id.backlightTxt);
				double backlight = (double)progress / seekBar.getMax();
				backlightTxt.setText(String.valueOf(backlight));
				ch.setBacklight(backlight);
			} catch (PhidgetException e) {
				e.printStackTrace();
			}
		}

		public void onStartTrackingTouch(SeekBar seekBar) {}

		public void onStopTrackingTouch(SeekBar seekBar) {}
	}

	private class contrastChangeListener implements SeekBar.OnSeekBarChangeListener {
		public void onProgressChanged(SeekBar seekBar, int progress,
									  boolean fromUser) {
			try {
				TextView contrastTxt = (TextView) findViewById(R.id.contrastTxt);
				double contrast = (double)progress / seekBar.getMax();
				contrastTxt.setText(String.valueOf(contrast));
				ch.setContrast(contrast);
			} catch (PhidgetException e) {
				e.printStackTrace();
			}
		}

		public void onStartTrackingTouch(SeekBar seekBar) {}

		public void onStopTrackingTouch(SeekBar seekBar) {}
	}

    class AttachEventHandler implements Runnable { 
    	Phidget ch;

		public AttachEventHandler(Phidget ch) {
			this.ch = ch;
		}

		public void run() {
			LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);
			settingsAndData.setVisibility(LinearLayout.VISIBLE);

			TextView attachedTxt = (TextView) findViewById(R.id.attachedTxt);

			attachedTxt.setText("Attached");
			try {
				TextView nameTxt = (TextView) findViewById(R.id.nameTxt);
				TextView serialTxt = (TextView) findViewById(R.id.serialTxt);
				TextView versionTxt = (TextView) findViewById(R.id.versionTxt);
				TextView channelTxt = (TextView) findViewById(R.id.channelTxt);
				TextView hubPortTxt = (TextView) findViewById(R.id.hubPortTxt);
				TextView labelTxt = (TextView) findViewById(R.id.labelTxt);

				nameTxt.setText(ch.getDeviceName());
				serialTxt.setText(Integer.toString(ch.getDeviceSerialNumber()));
				versionTxt.setText(Integer.toString(ch.getDeviceVersion()));
				channelTxt.setText(Integer.toString(ch.getChannel()));
				hubPortTxt.setText(Integer.toString(ch.getHubPort()));
				labelTxt.setText(ch.getDeviceLabel());

				TextView backlightTxt = (TextView) findViewById(R.id.backlightTxt);
				backlightTxt.setText(String.valueOf(((LCD)ch).getBacklight()));

				backlightBar.setProgress((int)(((LCD)ch).getBacklight() * backlightBar.getMax()));

				TextView contrastTxt = (TextView) findViewById(R.id.contrastTxt);
				contrastTxt.setText(String.valueOf(((LCD)ch).getContrast()));

				contrastBar.setProgress((int)(((LCD)ch).getContrast() * contrastBar.getMax()));

			} catch (PhidgetException e) {
				e.printStackTrace();
			}

			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }
    
    class DetachEventHandler implements Runnable {
    	Phidget ch;
    	
    	public DetachEventHandler(Phidget ch) {
    		this.ch = ch;
    	}
    	
		public void run() {
			LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);

			settingsAndData.setVisibility(LinearLayout.GONE);

			TextView attachedTxt = (TextView) findViewById(R.id.attachedTxt);
			attachedTxt.setText("Detached");

			TextView nameTxt = (TextView) findViewById(R.id.nameTxt);
			TextView serialTxt = (TextView) findViewById(R.id.serialTxt);
			TextView versionTxt = (TextView) findViewById(R.id.versionTxt);
			TextView channelTxt = (TextView) findViewById(R.id.channelTxt);
			TextView hubPortTxt = (TextView) findViewById(R.id.hubPortTxt);
			TextView labelTxt = (TextView) findViewById(R.id.labelTxt);

			nameTxt.setText(R.string.unknown_val);
			serialTxt.setText(R.string.unknown_val);
			versionTxt.setText(R.string.unknown_val);
			channelTxt.setText(R.string.unknown_val);
			hubPortTxt.setText(R.string.unknown_val);
			labelTxt.setText(R.string.unknown_val);

			((EditText)findViewById(R.id.text1)).setText("", TextView.BufferType.EDITABLE);
			((EditText)findViewById(R.id.text2)).setText("", TextView.BufferType.EDITABLE);
			
			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }

	class ErrorEventHandler implements Runnable {
		Phidget ch;
		ErrorEvent errorEvent;

		public ErrorEventHandler(Phidget ch, ErrorEvent errorEvent) {
			this.ch = ch;
			this.errorEvent = errorEvent;
		}

		public void run() {
			 if (errToast == null)
				 errToast = Toast.makeText(getApplicationContext(), errorEvent.getDescription(), Toast.LENGTH_SHORT);

			 //replace the previous toast message if a new error occurs
			 errToast.setText(errorEvent.getDescription());
			 errToast.show();
        }
	}

    @Override
    protected void onDestroy() {
    	super.onDestroy();
    	try {
			ch.close();

		} catch (PhidgetException e) {
			e.printStackTrace();
		}

		//Disable USB connection to Phidgets
    	if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
            com.phidget22.usb.Manager.Uninitialize();
    }

}

